#include "BallProject.h"
#include "AdaptStationary.h"
/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:
  
  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:
  F(int degree) : AbstractFunction<double, WorldVector<double> >(degree) {}

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dim = x.getSize();
    double r2 = x * x;
    double ux = exp(-10.0*r2);
    return -(400.0 * r2 - 20.0 * dim) * ux;
  }
};

Balldemo::Balldemo():
  ball("ball"),
  ballCenter(NULL),
  adaptInfo(NULL),
  adapt(NULL),
  matrixOperator(NULL),
  rhsOperator(NULL)
{
}

Balldemo::~Balldemo() {
  if(matrixOperator != NULL)
    delete matrixOperator;
  if(rhsOperator != NULL)
    delete rhsOperator;
  if(ballCenter != NULL)
    delete ballCenter;
  if(adapt != NULL)
    delete adapt;
  if(adaptInfo != NULL)
    delete adaptInfo;
}

void Balldemo::create(std::string& filename)  {
  // ===== init parameters =====
  Parameters::init(false, filename);
  ballCenter = new WorldVector< double >();
  ballCenter->set(0.0);

  // ===== create projection =====
  new AMDiS::BallProject(1, 
		  BOUNDARY_PROJECTION, 
		  *ballCenter, 
		  1.0);

  // ===== create and init the scalar problem ===== 
  ball.initialize(INIT_ALL);

  // === create adapt info ===
  adaptInfo = new AdaptInfo("ball->adapt");

  // === create adapt ===
  adapt = new AdaptStationary("ball->adapt", &ball, adaptInfo);
  
  // ===== create matrix operator =====
  matrixOperator=new Operator(ball.getFeSpace());
  matrixOperator->addSecondOrderTerm(new Laplace_SOT);
  ball.addMatrixOperator(matrixOperator);

  // ===== create rhs operator =====
  int degree = ball.getFeSpace()->getBasisFcts()->getDegree();
  rhsOperator = new Operator(ball.getFeSpace());
  rhsOperator->addZeroOrderTerm(new CoordsAtQP_ZOT(new F(degree)));
  ball.addVectorOperator(rhsOperator);

  // ===== add boundary conditions =====
  ball.addDirichletBC(1, new G);


}

int Balldemo::solve(SolutionInformation& solinfo) {
  assert(adaptInfo!=NULL);
  assert(adapt!=NULL);
  assert(matrixOperator!=NULL);
  assert(rhsOperator!=NULL);

  int retCode = adapt->adapt();
  solinfo.dofVec = ball.getSolution();
  //ball.writeFiles(adaptInfo,true);
  return retCode;
}

